from django.shortcuts import render
from .models import Test, Scan, ScanResults, ScanTests
from django.shortcuts import redirect
from django.utils import timezone
from django.http import HttpResponseRedirect
from src.tests.sql_injection import sql_injection
from src.tests.xss import xss
from src.tests.tampering import tampering
from src.tests.pollution import pollution
from src.tests.xml_injection import xml_injection
from src.tests.ssi_injection import ssi_injection
from src.tests.code_injection import code_injection
from django.contrib.auth import logout, authenticate, login
from django.contrib.auth.decorators import login_required


@login_required(login_url='login')
def index(request):
    tests = Test.objects.order_by('pub_date')
    datos = {'tests': tests}
    return render(request, 'scanner/index.html', datos)


@login_required(login_url='login')
def scan(request):
    # if request.user.is_authenticated() is False:
    #     return redirect('login')

    resultat = {}

    tests = Test.objects.all()
    url = request.POST.get('url', False)
    ids = request.POST.get('ids', None)

    clases = request.POST['class']
    params = request.POST['param']
    posts = request.POST['post']
    resultat = {
        'tests': [],
        'parametres': {
            'url': url,
            'ids': ids,
            'clases': clases,
            'params': params,
            'posts': posts
        }
    }

    # Afegim a la BBDD - Scan
    new_scan = Scan(
        user=request.user,
        url=url,
        posts=posts,
        ids=ids,
        clases=clases,
        params=params,
        pub_date=timezone.now()
    )
    new_scan.save()

    # Mirem quins tests s'han especificat
    for test in tests:
        if 'test'+str(test.id) in request.POST:

            # Afegim a la BBDD - ScanTests
            new_test = ScanTests(
                idscan=new_scan,
                test=test
            )
            new_test.save()
            scan_res = []

            if test.id == 5:
                scan_res = sql_injection.scan(url, params=params, ids=ids, clases=clases, posts=posts)
            elif test.id == 2:
                scan_res = xss.scan(url, params=params, ids=ids, clases=clases, posts=posts)
            elif test.id == 3:
                scan_res = tampering.scan(url, params=params, ids=ids, clases=clases, posts=posts)
            elif test.id == 4:
                scan_res = pollution.scan(url, params=params, ids=ids, clases=clases, posts=posts)
            elif test.id == 8:
                scan_res = xml_injection.scan(url, params=params, ids=ids, clases=clases, posts=posts)
            elif test.id == 9:
                scan_res = ssi_injection.scan(url, params=params, ids=ids, clases=clases, posts=posts)
            elif test.id == 12:
                scan_res = code_injection.scan(url, params=params, ids=ids, clases=clases, posts=posts)
            scan_info = {
                'vulnerabilitats': scan_res,
                'test': test.test
            }
            resultat['tests'].append(scan_info)

            # Recorro els resultats i els guardo en BBDD
            if scan_res:
                for lista in scan_res:
                    for param, val in lista.items():
                        for fields in val:
                            vul = ScanResults(
                                idscanTest=new_test,
                                param=param,
                                vulnerability=fields['type'],
                                description=fields['title'],
                                payload=fields['payload'],
                                pub_date=timezone.now()
                            )
                            vul.save()
    return render(request, 'scanner/result.html', resultat)
    # return JsonResponse(sqlin, safe=False)


def login_vista(request):
    return render(request, 'scanner/login.html')


def login_action(request):
    user = request.POST['user']
    passwd = request.POST['password']
    user = authenticate(username=user, password=passwd)
    if user is not None and user.is_active:
        login(request, user)
        return redirect('index')
    else:
        return redirect('login')


def logout_action(request):
    logout(request)
    return render(request, 'scanner/login.html')


@login_required(login_url='login')
def myscans(request):
    data = {
        'scans': {}
    }
    scans = Scan.objects.filter(user=request.user)
    # scans = ScanResults.objects.filter(idscanTest__idscan__user=request.user)
    for scan in scans:
        vul_total = 0
        tests = ScanTests.objects.filter(idscan=scan.id)
        test_total = len(tests)
        for test in tests:
            vul = len(ScanResults.objects.filter(idscanTest=test.id))
            vul_total += vul

        # data['scans'].append({
        #     'url': scan.url,
        #     'id': scan.id,
        #     'data': scan.pub_date,
        #     'tests': test_total,
        #     'vul': vul_total
        # })

        if scan.url not in data['scans']:
            data['scans'][scan.url] = []

        data['scans'][scan.url].append({
            'url': scan.url,
            'id': scan.id,
            'data': scan.pub_date,
            'tests': test_total,
            'vul': vul_total
        })

    return render(request, 'scanner/myscans.html', data)


def info(request):
    return render(request, 'scanner/informacio.html')


@login_required(login_url='login')
def scan_detail(request, scan_id):

    scan = Scan.objects.get(pk=scan_id)
    tests = ScanTests.objects.filter(idscan=scan_id)

    resultat = {
        'tests': [],
        'parametres': {
            'url': scan.url,
            'ids': scan.ids,
            'clases': scan.clases,
            'params': scan.params,
            'posts': scan.posts
        }
    }

    for test in tests:
        # print 'id', test.id
        params = ScanResults.objects.filter(idscanTest=test.id)
        scanResult = {}
        for param in params:
            if param.param not in scanResult:
                scanResult[param.param] = []
            scanResult[param.param].append({
                'type': param.vulnerability,
                'payload': param.payload,
                'title': param.description
            })

        scan_info = {
            'vulnerabilitats': [scanResult],
            'test': test.test
        }

        resultat['tests'].append(scan_info)
        # print 'resultat', resultat

    return render(request, 'scanner/result.html', resultat)